/* JOH data interpreter.
   Copyright (C) 2011 Sergey Poznyakoff

   JOH is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   JOH is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JOH.  If not, see <http://www.gnu.org/licenses/>. */

#include "joh.h"

struct interpreter {
	int (*decode) (void **state, struct joh_param param[joh_param_max],
		       joh_cbuf_t ibuf, const char *id);
	int (*encode) (struct joh_obuf *buf, size_t count,
		       const int *param_idx, struct joh_param *param_vals);

};

static int
decode_error(void **state JOH_ARG_UNUSED,
	     struct joh_param param[joh_param_max] JOH_ARG_UNUSED,
	     joh_cbuf_t ibuf, const char *id)
{
	char *pfx[2] = { (char*) id, NULL };

	error("%s: INTERNAL ERROR: wrong client class while decoding", id);
	joh_logdump(pfx, joh_cbuf_finish(ibuf), joh_cbuf_size(ibuf));
	return -1;
}

static int
encode_error(struct joh_obuf *buf, size_t count,
	     const int *param_idx, struct joh_param *param_vals)
{
	error("INTERNAL ERROR: wrong client class while encoding");
	return -1;
}
	

static struct interpreter interpreter[] = {
	{ decode_error, encode_error },
	{ joh_pdu_decode, joh_pdu_encode },
	{ joh_http_decode, joh_http_encode }
};

int
joh_data_decode(struct joh_session *sess, struct joh_sockaddr *sa)
{
	int rc;

	rc = interpreter[sa->class].decode(&sess->decstate, sess->param,
					   sess->ibuf, joh_sockaddr_str(sa));
	if (rc == 0) {
		if (!sess->param[joh_param_id].ptr) {
			error("no ID in %s packet from %s",
			      client_class_str[sa->class],
			      joh_sockaddr_str(sa));
			return -1;
		}
		if (!sess->param[joh_param_key].ptr) {
			error("no key in %s packet from %s",
			      client_class_str[sa->class],
			      joh_sockaddr_str(sa));
			return -1;
		}
	}
	return rc;
}

int
joh_data_encode(struct joh_sockaddr *sa,
		struct joh_obuf *buf, size_t count, const int *param_idx,
		struct joh_param *param_vals)
{
	return interpreter[sa->class].encode(buf, count, param_idx,
					     param_vals);
}
